// Package.cpp: implementation of the Package class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Package.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

const int PACKAGE_SERIAL_NO = 1;

class CObjectBufConverter 
{
public:
    static PackSize ToBuffer(CObject * pObj, BYTE *& pBuf)
    {
        if (pObj && pObj->IsSerializable()) {
            CMemFile memFile;
            CArchive ar(&memFile, CArchive::store);
            pObj->Serialize(ar);
            ar.Flush();
            ar.Close();
            memFile.SeekToBegin();
            pBuf = new BYTE[(UINT)memFile.GetLength()];
            PackSize size = memFile.Read(pBuf, (UINT)memFile.GetLength());
            return size;
        }
        return -1;
    }
    static PackSize FromBuffer(const BYTE * pBuf, PackSize size, CObject * pObj)
    {
        if (pObj && pObj->IsSerializable()) {
            if (size > 0) {
                CMemFile memFile;
                memFile.Write(pBuf, size);
                memFile.SeekToBegin();
                CArchive ar(&memFile, CArchive::load);
                pObj->Serialize(ar);
                ar.Flush();
                ar.Close();

                PackSize bytesUsed = (UINT)memFile.GetPosition();
                return bytesUsed;
            }
        }
        return -1;
    }
};

IMPLEMENT_SERIAL(SerialToBuf, CObject, PACKAGE_SERIAL_NO);
SerialToBuf::SerialToBuf()
{
}

SerialToBuf::~SerialToBuf()
{
}

void SerialToBuf::Serialize(CArchive & ar)
{
    CObject::Serialize(ar);
}
   
PackSize SerialToBuf::ToBuffer(BYTE *& pBuf) 
{
    return CObjectBufConverter::ToBuffer(this, pBuf);
}

PackSize SerialToBuf::FromBuffer(BYTE * pBuf, PackSize size)
{
    return CObjectBufConverter::FromBuffer(pBuf, size, this);
}

//////////////////////////////////////////////////////////////////////////////////////////
// PackObj
//    An individual entry within the Package CMap. Each entry within a Package must be 
//    serializable.
//
IMPLEMENT_SERIAL(PackObj, CObject, PACKAGE_SERIAL_NO);
PackObj::PackObj()
{
    pBuf = NULL;
}

PackObj::~PackObj()
{
    delete[] pBuf;
    pBuf = NULL;
}

PackObj & PackObj::operator=(const PackObj& obj)
{
    delete[] pBuf;
    pBuf = NULL;
    bufSize = obj.CopyBuf(pBuf);
    return *this;
}

PackSize PackObj::CopyBuf(BYTE *& pBuffer) const
{
    pBuffer = new BYTE[bufSize];
    memcpy(pBuffer, pBuf, bufSize);
    return bufSize;
}

void PackObj::Serialize(CArchive & ar)
{
    SerialToBuf::Serialize(ar);

    if (ar.IsStoring()) {
        ar << (int)sType;
        ar << name;
        ar << bufSize;
        for (unsigned int i = 0; i < bufSize; i++) {
            BYTE byte = (BYTE)pBuf[i];
            ar << byte;
        }
    } else {
        int sTypeInt;
        ar >> sTypeInt;
        ar >> name;
        sType = (StoreType)sTypeInt;
        ar >> bufSize;
        delete[] pBuf;
        pBuf = new BYTE[bufSize];
        for (unsigned int i = 0; i < bufSize; i++) {
            BYTE byte;
            ar >> byte;
            pBuf[i] = byte;
        }
    }
}

bool PackObj::Get(CString & objName, CObject * pObj)
{
    objName = name;
    if (sType == P_OBJECT) {
        if (CObjectBufConverter::FromBuffer(pBuf, bufSize, pObj) >= 0)
            return true;
    }
    return false;
}

bool PackObj::Set(const CString & objName, CObject * pObj)
{
    name = objName;
    sType = P_OBJECT;
    delete pBuf;
    pBuf = NULL;
    bufSize = CObjectBufConverter::ToBuffer(pObj, pBuf);
    if (bufSize >= 0)
        return true;
    return false;
}

bool PackObj::Get(CString & objName, CString & str)
{
    objName = name;
    if (sType == P_STRING) {
        str = (char*)pBuf;
        return true;
    }
    return false;
}

bool PackObj::Set(const CString & objName, const CString & str)
{
    name = objName;
    sType = P_STRING;
    delete pBuf;
    pBuf = NULL;
    bufSize = str.GetLength() + 1;
    pBuf = new BYTE[bufSize];
    if (pBuf) {
        strcpy((char*)pBuf, (LPCSTR)str);
        return true;
    }
    return false;
}

bool PackObj::Get(CString & objName, ValueBuf & vb)
{
    objName = name;
    if (sType == P_64 && pBuf && bufSize == 8) {
        memcpy(vb.pBuf, pBuf, bufSize);
        return true;
    }
    return false;    
}

bool PackObj::Set(const CString & objName, const ValueBuf & vb)
{
    name = objName;
    sType = P_64;
    delete pBuf;
    pBuf = NULL;
    bufSize = 8;
    pBuf = new BYTE[bufSize];
    memcpy(pBuf, vb.pBuf, bufSize);
    return true;
}


//////////////////////////////////////////////////////////////////////////////////////////
// Package
//    Used to package up many objects into one object that can be easily Serialized or
//    converted to a void * pBuf.
//
IMPLEMENT_SERIAL(Package, SerialToBuf, PACKAGE_SERIAL_NO);
Package::Package()
{
}

Package::~Package()
{
    freeMemory();
}

Package & Package::operator=(const Package & pack)
{
    Copy(pack);
    return *this;
}

void Package::Copy(const Package & pack)
{
    freeMemory();

    POSITION pos = pack.map.GetStartPosition();
    while(pos) {
        CString objName;
        PackObj * pObj;
        pack.map.GetNextAssoc(pos, objName, pObj);
        if (pObj) {
            PackObj * pStoreObj = new PackObj;
            BYTE * pBuf;
            PackSize size = CObjectBufConverter::ToBuffer(pObj, pBuf);
            CObjectBufConverter::FromBuffer(pBuf, size, pStoreObj);
            delete[] pBuf;
            map.SetAt(objName, pStoreObj);
        }
    }
}

void Package::Serialize(CArchive & ar)
{
    SerialToBuf::Serialize(ar);
    if (ar.IsStoring()) {
        int mapCount = (int)map.GetCount();
        ar << mapCount;
        POSITION pos = map.GetStartPosition();
        while (pos) {
            CString objName;
            PackObj * pStore;
            map.GetNextAssoc(pos, objName, pStore);
            ASSERT(pStore);
            if (pStore)
                pStore->Serialize(ar);
        }
    } else {
        freeMemory();
        int mapCount;
        ar >> mapCount;
        for (int i = 0; i < mapCount; i++) {
            PackObj * pStore = new PackObj();
            pStore->Serialize(ar);
            map.SetAt(pStore->Name(), pStore);
        }
    }
}

void Package::freeMemory()
{
    POSITION pos = map.GetStartPosition();
    while (pos) {
        CString objName;
        PackObj * pStore;
        map.GetNextAssoc(pos, objName, pStore);
        ASSERT(pStore);
        if (pStore)
            delete pStore;
    }
    map.RemoveAll();
}

// get CObject
bool Package::Get(const CString & objName, CObject * pObj) const
{
    if (pObj && pObj->IsSerializable()) {
        PackObj * pStore;
        if (map.Lookup(objName, pStore)) {
            if (pStore) {
                CString objName2 = objName;
                if (pStore->Get(objName2, pObj))
                    return true;
            }
        }
    }
    return false;
}

// set CObject
bool Package::Set(const CString & objName, CObject * pObj)
{
    if (pObj && pObj->IsSerializable()) {
        PackObj * pNewObj = new PackObj();
        if (pNewObj->Set(objName, pObj)) {
            map.SetAt(objName, pNewObj);
            return true;
        }
    }
    return false;
}

// get string
bool Package::Get(const CString & objName, CString & str) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            if (pStore->Get(objName2, str))
                return true;
        }
    }
    return false;
}

// set string
bool Package::Set(const CString & objName, const CString & str)
{
    PackObj * pStore = new PackObj;
    if (pStore->Set(objName, str)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get uint64
bool Package::Get(const CString & objName, unsigned __int64 & n64) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n64 = vb.d_uint64;
                return true;
            }
        }
    }
    return false;            
}

// set uint64
bool Package::Set(const CString & objName, unsigned __int64 n64)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_uint64 = n64;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get int64
bool Package::Get(const CString & objName, __int64 & n64) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n64 = vb.d_int64;
                return true;
            }
        }
    }
    return false;
}

// set int64
bool Package::Set(const CString & objName, __int64 n64)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_int64 = n64;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get double
bool Package::Get(const CString & objName, double & dnum) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                dnum = vb.d_fp64;
                return true;
            }
        }
    }
    return false;
}

// set double
bool Package::Set(const CString & objName, double dnum)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_fp64 = dnum;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get uint
bool Package::Get(const CString & objName, UINT & n32) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n32 = vb.d_uint32;
                return true;
            }
        }
    }
    return false;
}

// set uint
bool Package::Set(const CString & objName, UINT n32)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_uint32 = n32;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get int
bool Package::Get(const CString & objName, int & n32) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n32 = vb.d_int32;
                return true;
            }
        }
    }
    return false;    
}

// set int
bool Package::Set(const CString & objName, int n32)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_int32 = n32;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get float
bool Package::Get(const CString & objName, float & fnum) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                fnum = vb.d_fp32;
                return true;
            }
        }
    }
    return false;    
}

// set float
bool Package::Set(const CString & objName, float fnum)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_fp32 = fnum;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get word (ushort)
bool Package::Get(const CString & objName, unsigned short & n16) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n16 = vb.d_uint16;
                return true;
            }
        }
    }
    return false;
}

// set word (ushort)
bool Package::Set(const CString & objName, unsigned short n16)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_uint16 = n16;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get short
bool Package::Get(const CString & objName, short & n16) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n16 = vb.d_int16;
                return true;
            }
        }
    }
    return false;
}

// set short
bool Package::Set(const CString & objName, short n16)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_int16 = n16;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// Get byte
bool Package::Get(const CString & objName, BYTE & n8) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n8 = vb.d_byte;
                return true;
            }
        }
    }
    return false;
}

// set byte
bool Package::Set(const CString & objName, BYTE n8)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_byte = n8;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}

// get char
bool Package::Get(const CString & objName, char & n8) const
{
    PackObj * pStore;
    if (map.Lookup(objName, pStore)) {
        if (pStore) {
            CString objName2 = objName;
            PackObj::ValueBuf vb;
            if (pStore->Get(objName2, vb)) {
                n8 = vb.d_char;
                return true;
            }
        }
    }
    return false;
}

// set char
bool Package::Set(const CString & objName, char n8)
{
    PackObj * pStore = new PackObj;
    PackObj::ValueBuf vb;
    vb.d_char = n8;
    if (pStore->Set(objName, vb)) {
        map.SetAt(objName, pStore);
        return true;
    }
    return false;
}