// UserList.cpp : implementation file
//

#include "stdafx.h"
#include "MailSlotApp.h"
#include "UserList.h"
#include "mailslot.h"
#include "mailslotdlg.h"

#include <Afxtempl.h> // for carray

const CString USER_FILENAME = "userary.dat";
const CString LOGIN_MSG = "LOGIN";

IMPLEMENT_SERIAL(UserAry, CObject, 1);

UserAry::UserAry()
{ ; }

UserAry::~UserAry()
{ 
    entries.RemoveAll(); 
}

void UserAry::Serialize(CArchive & ar)
{
    if (ar.IsStoring()) {
        ar << (int)entries.GetSize();
        for (int i = 0; i < entries.GetSize(); i++) {
            UserEntry entry;
            if (Get(i, entry)) {
                ar << entry.computerName << entry.userName;
            }
        }
    } else {
        int count;
        ar >> count;
        for (int i = 0; i < count; i++) {
            UserEntry entry;
            ar >> entry.computerName >> entry.userName;
            Add(entry);
        }
    }
}

bool UserAry::Get(int index, UserEntry & entry)
{
    if (index < Count()) {
        entry = entries.GetAt(index);
        return true;
    }
    return false;
}

int UserAry::Count()
{
    return (int)entries.GetCount();
}

void UserAry::Add(UserEntry & entry)
{
    int loc = Find(entry.userName);
    if (loc == -1) {
        entries.Add(entry);
    } else {
        entries.SetAt(loc, entry);
    }
}

int UserAry::Find(const CString & userName)
{
    // should use a cmap to improve speed later
    for (int i = 0; i < Count(); i++) {
        UserEntry entry;
        if (Get(i, entry))
            if (entry.userName == userName)
                return i;
    }
    return -1;
}

void UserAry::Del(int index)
{
    if (index >= 0 && index < Count()) {
        entries.RemoveAt(index);
    }
}

bool UserAry::load()
{
    CFile f;
    if (f.Open(USER_FILENAME, CFile::modeRead)) {
        if (f.GetLength() > 0) {
            CArchive ar(&f, CArchive::load);
            Serialize(ar);
            ar.Close();
        }
        f.Close();
        return true;
    }
    return false;
}

bool UserAry::store()
{
    CFile f;
    if (f.Open(USER_FILENAME, CFile::modeCreate|CFile::modeReadWrite)) {
        CArchive ar(&f, CArchive::store);
        Serialize(ar);
        ar.Flush();
        ar.Close();
        f.Close();
        return true;
    }
    return false;
}


// UserList dialog

IMPLEMENT_DYNAMIC(UserList, CDialog)
UserList::UserList(const CString & userName, CWnd* pParent /*=NULL*/)
	: CDialog(UserList::IDD, pParent)
{
    m_userName = userName;
    ASSERT(m_userName.IsEmpty() == FALSE);
    pParentWnd = pParent;
}

UserList::~UserList()
{
}

void UserList::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_USER_LIST, m_list);
}

BOOL UserList::OnInitDialog()
{
    BOOL ret = CDialog::OnInitDialog();

    if (m_userName.IsEmpty()) {
        AfxMessageBox("You must supply a user name at the command prompt\n'MailSlotApp Chad'");
        exit(0);
    }

    SetWindowText("IM for " + m_userName);

    m_mailSlot.Create(m_userName, this, MAILSLOT_NOTIFY_MSG);

    m_userAry.load();

    UserEntry entry;
    entry.userName = m_userName;
    entry.computerName = m_mailSlot.ComputerName();

    int loc = m_userAry.Find(m_userName);
    if (loc == -1) {
        m_userAry.Add(entry);
        m_userAry.store();
    } else {
        m_userAry.Del(loc);
        m_userAry.Add(entry);
    }

    RefreshList();
    NotifyOtherUsers();

    return ret;
}

BEGIN_MESSAGE_MAP(UserList, CDialog)
    ON_LBN_SELCHANGE(IDC_USER_LIST, OnLbnSelchangeUserList)
    ON_LBN_DBLCLK(IDC_USER_LIST, OnLbnDblclkUserList)
    ON_MESSAGE(MAILSLOT_SEND_MSG, OnMailSlotSendMsg)
    ON_MESSAGE(MAILSLOT_NOTIFY_MSG, OnMailSlotNotify)
    ON_WM_DESTROY()
END_MESSAGE_MAP()


// UserList message handlers

void UserList::OnLbnSelchangeUserList()
{
}

void UserList::OnLbnDblclkUserList()
{
    int sel = m_list.GetCurSel();
    if (sel >= 0 && sel < m_list.GetCount()) {
        CString text;
        m_list.GetText(sel, text);
        UserEntry * e = (UserEntry*)m_list.GetItemData(sel);
        if (e) {
            MailSlotDlg * pWnd = GetDlg(e->computerName, e->userName);
        }
    }
}

void UserList::OnDestroy()
{
    ClearList();
    CDialog::OnDestroy();
    m_userAry.load();
    int loc = m_userAry.Find(m_userName);
    if (loc >= 0)
        m_userAry.Del(loc);
    m_userAry.store();
}

void UserList::NotifyOtherUsers()
{
    for (int i = 0; i < m_userAry.Count(); i++) {
        UserEntry entry;
        if (m_userAry.Get(i, entry)) {
            MailSlotMessage msg;
            msg.MsgType(MailSlotMessage::SYSTEM);
            msg.MsgText(LOGIN_MSG);
            msg.ToComputerName(entry.computerName);
            msg.ToUserName(entry.userName);
            msg.FromComputerName(m_mailSlot.ComputerName());
            msg.FromUserName(m_userName);
            m_mailSlot.Send(msg);
        }
    }
}

void UserList::ClearList()
{
    for (int k = 0; k < m_list.GetCount(); k++) {
        UserEntry * e = (UserEntry*)m_list.GetItemData(k);
        delete e;
    }
    m_list.ResetContent();
}

void UserList::RefreshList()
{
    ClearList();
    m_userAry.load();
    for (int i = 0; i < m_userAry.Count(); i++) {
        UserEntry * entry = new UserEntry;
        if (m_userAry.Get(i, *entry)) {
            m_list.InsertString(i, entry->computerName + ":" + entry->userName);
            m_list.SetItemData(i, (DWORD_PTR)entry);
        }       
    }
}

MailSlotDlg * UserList::GetDlg(const CString & toComputerName, const CString & toUserName)
{
    MailSlotDlg * pWnd = NULL;
    if (dlgMap.Lookup(toUserName, pWnd)) {
        pWnd->ShowWindow(SW_SHOW);
        pWnd->BringWindowToTop();
        return pWnd;
    } else {
        pWnd = new MailSlotDlg(m_userName, toComputerName, toUserName, this);
        if (pWnd) {
            pWnd->Create(IDD_MAILSLOTAPP_DIALOG);            
            dlgMap.SetAt(toUserName, pWnd);
            return pWnd;
        }
    }
    return NULL;
}

LRESULT UserList::OnMailSlotSendMsg(WPARAM w, LPARAM l)
{
   MailSlotMessage * pMsg = (MailSlotMessage*)w;
   pMsg->FromUserName(m_userName);
   pMsg->FromComputerName(m_mailSlot.ComputerName());   
   m_mailSlot.Send(*pMsg);
   return 1;
}

LRESULT UserList::OnMailSlotNotify(WPARAM w, LPARAM l)
{
    MailSlotMessage msg;
    while (m_mailSlot.Read(msg)) {
        if (msg.MsgType() == MailSlotMessage::TEXT) {
            MailSlotDlg * pWnd = GetDlg(msg.FromComputerName(), msg.FromUserName());        
            if (pWnd)
                pWnd->AddMessageToHistory(msg); // add message to history
        } else {
            if (msg.MsgText() == LOGIN_MSG) {
                RefreshList();
            }
        }
    }
    return 1;
}